@@ -79,7 +79,7 @@ void FileBrowserProxyModel::listRecursiveNewEntries(KIO::Job *job, const KIO::UD
         return (sortOrder() == Qt::AscendingOrder) ? (first < second) : (first > second);
     });
 
-    for (const auto &oneEntry : vNewEntries) {
+    for (const auto &oneEntry : std::as_const(vNewEntries)) {
         auto fullPath = QString{};
         auto fullPathUrl = QUrl{};
 
@@ -127,11 +127,9 @@ void FileBrowserProxyModel::genericEnqueueToPlayList(const QModelIndex &rootInde
     recursiveEnqueue();
 }
 
-void FileBrowserProxyModel::enqueueToPlayList(const QModelIndex &rootIndex)
+void FileBrowserProxyModel::enqueueAll(ElisaUtils::PlayListEnqueueMode enqueueMode, ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
 {
-    genericEnqueueToPlayList(rootIndex,
-                             ElisaUtils::AppendPlayList,
-                             ElisaUtils::DoNotTriggerPlay);
+    genericEnqueueToPlayList(QModelIndex{}, enqueueMode, triggerPlay);
 }
 
 void FileBrowserProxyModel::enqueue(const DataTypes::MusicDataType &newEntry,
@@ -187,13 +185,6 @@ void FileBrowserProxyModel::enqueue(const DataTypes::MusicDataType &newEntry,
     recursiveEnqueue();
 }
 
-void FileBrowserProxyModel::replaceAndPlayOfPlayList(const QModelIndex &rootIndex)
-{
-    genericEnqueueToPlayList(rootIndex,
-                             ElisaUtils::ReplacePlayList,
-                             ElisaUtils::TriggerPlay);
-}
-
 void FileBrowserProxyModel::disconnectPlayList()
 {
     if (mPlayList) {
@@ -212,6 +203,12 @@ void FileBrowserProxyModel::connectPlayList()
 
 void FileBrowserProxyModel::recursiveEnqueue()
 {
+    if (mPendingEntries.empty()) {
+        mEnqueueInProgress = false;
+        Q_EMIT entriesToEnqueue(mAllData, mEnqueueMode, mTriggerPlay);
+        return;
+    }
+
     auto [rootUrl, isDirectory] = mPendingEntries.front();
 
     if (rootUrl.isEmpty()) {
@@ -232,13 +229,6 @@ void FileBrowserProxyModel::recursiveEnqueue()
             if (!ElisaUtils::isPlayList(mMimeDatabase.mimeTypeForUrl(rootUrl))) {
                 mAllData.push_back({{{DataTypes::ElementTypeRole, ElisaUtils::FileName}, {DataTypes::ResourceRole, rootUrl}}, rootUrl.toString(), rootUrl});
             }
-
-            if (mPendingEntries.empty()) {
-                mEnqueueInProgress = false;
-                Q_EMIT entriesToEnqueue(mAllData, mEnqueueMode, mTriggerPlay);
-                return;
-            }
-
             recursiveEnqueue();
         }
     }
