@@ -15,22 +15,20 @@
 #include <algorithm>
 
 #include <KFormat>
+#include <QDir>
+#include <QFileInfo>
+
+using namespace Qt::Literals::StringLiterals;
 
 QList<DataTypes::ColumnsRoles> displayFields(const ElisaUtils::PlayListEntryType dataType)
 {
     switch (dataType) {
     case ElisaUtils::Track:
         return {
-            DataTypes::TitleRole, DataTypes::ArtistRole,
-            DataTypes::AlbumRole, DataTypes::AlbumArtistRole,
-            DataTypes::TrackNumberRole, DataTypes::DiscNumberRole,
-            DataTypes::RatingRole, DataTypes::GenreRole,
-            DataTypes::LyricistRole, DataTypes::ComposerRole,
-            DataTypes::CommentRole, DataTypes::YearRole,
-            DataTypes::ChannelsRole, DataTypes::BitRateRole,
-            DataTypes::SampleRateRole, DataTypes::LastPlayDate,
-            DataTypes::PlayCounter, DataTypes::DurationRole,
-            DataTypes::LyricsRole
+            DataTypes::TitleRole,      DataTypes::ArtistRole,  DataTypes::AlbumRole,    DataTypes::AlbumArtistRole, DataTypes::TrackNumberRole,
+            DataTypes::DiscNumberRole, DataTypes::RatingRole,  DataTypes::GenreRole,    DataTypes::LyricistRole,    DataTypes::ComposerRole,
+            DataTypes::CommentRole,    DataTypes::YearRole,    DataTypes::ChannelsRole, DataTypes::BitRateRole,     DataTypes::SampleRateRole,
+            DataTypes::LastPlayDate,   DataTypes::PlayCounter, DataTypes::DurationRole, DataTypes::LyricsRole,
         };
     case ElisaUtils::Radio:
         return {DataTypes::TitleRole, DataTypes::ResourceRole, DataTypes::CommentRole, DataTypes::ImageUrlRole};
@@ -42,8 +40,7 @@ QList<DataTypes::ColumnsRoles> displayFields(const ElisaUtils::PlayListEntryType
 TrackMetadataModel::TrackMetadataModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    connect(&mLyricsValueWatcher, &QFutureWatcher<QString>::finished,
-            this, &TrackMetadataModel::lyricsValueIsReady);
+    connect(&mLyricsValueWatcher, &QFutureWatcher<std::pair<QString, QString>>::finished, this, &TrackMetadataModel::lyricsValueIsReady);
 }
 
 TrackMetadataModel::~TrackMetadataModel()
@@ -226,6 +223,9 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
             break;
         }
         break;
+    case HasDataRole:
+        result = mDisplayData.contains(currentKey);
+        break;
     }
 
     return result;
@@ -238,7 +238,7 @@ bool TrackMetadataModel::setData(const QModelIndex &index, const QVariant &value
 
         mDisplayData[dataType] = value;
 
-        Q_EMIT dataChanged(index, index, QVector<int>() << role);
+        Q_EMIT dataChanged(index, index, QList<int>() << role);
         return true;
     }
     return false;
@@ -246,7 +246,13 @@ bool TrackMetadataModel::setData(const QModelIndex &index, const QVariant &value
 
 void TrackMetadataModel::saveChanges()
 {
-    mFullData.insert(mDisplayData);
+    for (const auto role : mDisplayKeys) {
+        if (mDisplayData.contains(role)) {
+            mFullData[role] = mDisplayData[role];
+        } else {
+            mFullData.remove(role);
+        }
+    }
 }
 
 void TrackMetadataModel::undoChanges()
@@ -260,6 +266,7 @@ QHash<int, QByteArray> TrackMetadataModel::roleNames() const
 
     names[ItemNameRole] = "name";
     names[ItemTypeRole] = "type";
+    names[HasDataRole] = "hasData";
 
     return names;
 }
@@ -292,6 +299,11 @@ QString TrackMetadataModel::lyrics() const
     return mFullData[TrackDataType::key_type::LyricsRole].toString();
 }
 
+QString TrackMetadataModel::lyricsLocation() const
+{
+    return mFullData[TrackDataType::key_type::LyricsLocationRole].toString();
+}
+
 qulonglong TrackMetadataModel::databaseId() const
 {
     return mDatabaseId;
@@ -334,14 +346,12 @@ void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDa
 
 void TrackMetadataModel::resetDisplayData()
 {
-    const auto fieldsForTrack = displayFields(mFullData.elementType());
     beginResetModel();
     mDisplayData.clear();
-    mDisplayKeys.clear();
+    mDisplayKeys = displayFields(mFullData.elementType());
 
-    for (DataTypes::ColumnsRoles role : fieldsForTrack) {
+    for (DataTypes::ColumnsRoles role : mDisplayKeys) {
         if (mFullData.constFind(role) != mFullData.constEnd()) {
-            mDisplayKeys.push_back(role);
             mDisplayData[role] = mFullData[role];
         }
     }
@@ -366,15 +376,17 @@ void TrackMetadataModel::removeMetaData(DataTypes::ColumnsRoles metaData)
 
 TrackMetadataModel::TrackDataType::mapped_type TrackMetadataModel::dataFromType(TrackDataType::key_type metaData) const
 {
-    return mFullData[metaData];
+    return mDisplayData[metaData];
 }
 
 void TrackMetadataModel::fillLyricsDataFromTrack()
 {
-    beginInsertRows({}, mDisplayData.size(), mDisplayData.size());
-    mDisplayKeys.push_back(DataTypes::LyricsRole);
-    mDisplayData[DataTypes::LyricsRole] = mLyricsValueWatcher.result();
-    endInsertRows();
+    const auto result = mLyricsValueWatcher.result();
+    mDisplayData[DataTypes::LyricsRole] = result.first;
+    mDisplayData[DataTypes::LyricsLocationRole] = result.second;
+    // This is safe as long as we keep the lyrics as the final tag
+    const auto lyricsIndex = index(mDisplayKeys.size() - 1);
+    Q_EMIT dataChanged(lyricsIndex, lyricsIndex);
 }
 
 const TrackMetadataModel::TrackDataType &TrackMetadataModel::allTrackData() const
@@ -384,10 +396,12 @@ const TrackMetadataModel::TrackDataType &TrackMetadataModel::allTrackData() cons
 
 void TrackMetadataModel::lyricsValueIsReady()
 {
-    if (!mLyricsValueWatcher.result().isEmpty()) {
+    auto result = mLyricsValueWatcher.result();
+    if (!result.first.isEmpty()) {
         fillLyricsDataFromTrack();
 
-        mFullData[DataTypes::LyricsRole] = mLyricsValueWatcher.result();
+        mFullData[DataTypes::LyricsRole] = result.first;
+        mFullData[DataTypes::LyricsLocationRole] = result.second;
 
         Q_EMIT lyricsChanged();
     }
@@ -460,54 +474,30 @@ DataTypes::ColumnsRoles TrackMetadataModel::trackKey(int index) const
 
 void TrackMetadataModel::removeDataByIndex(int index)
 {
-    auto dataKey = mDisplayKeys[index];
-
-    mDisplayData[dataKey] = {};
-    mDisplayKeys.removeAt(index);
-}
-
-void TrackMetadataModel::addDataByName(const QString &name)
-{
-    DataTypes::ColumnsRoles newRole = DataTypes::TitleRole;
-
-    if (name == i18nc("@label:textbox Track title for track metadata view", "Title")) {
-        newRole = DataTypes::TitleRole;
-    } else if (name == i18nc("@label:textbox Track artist for track metadata view", "Artist")) {
-        newRole = DataTypes::ArtistRole;
-    } else if (name == i18nc("@label:textbox Album name for track metadata view", "Album")) {
-        newRole = DataTypes::AlbumRole;
-    } else if (name == i18nc("@label:textbox Album artist for track metadata view", "Album Artist")) {
-        newRole = DataTypes::AlbumArtistRole;
-    } else if (name == i18nc("@label:textbox Track number for track metadata view", "Track Number")) {
-        newRole = DataTypes::TrackNumberRole;
-    } else if (name == i18nc("@label:textbox Disc number for track metadata view", "Disc Number")) {
-        newRole = DataTypes::DiscNumberRole;
-    } else if (name == i18nc("@label:textbox Rating label for information panel", "Rating")) {
-        newRole = DataTypes::RatingRole;
-    } else if (name == i18nc("@label:textbox Genre label for track metadata view", "Genre")) {
-        newRole = DataTypes::GenreRole;
-    } else if (name == i18nc("@label:textbox Lyricist label for track metadata view", "Lyricist")) {
-        newRole = DataTypes::LyricistRole;
-    } else if (name == i18nc("@label:textbox Composer name for track metadata view", "Composer")) {
-        newRole = DataTypes::ComposerRole;
-    } else if (name == i18nc("@label:textbox Comment label for track metadata view", "Comment")) {
-        newRole = DataTypes::CommentRole;
-    } else if (name == i18nc("@label:textbox Year label for track metadata view", "Year")) {
-        newRole = DataTypes::YearRole;
-    } else if (name == i18nc("@label:textbox Channels label for track metadata view", "Channels")) {
-        newRole = DataTypes::ChannelsRole;
-    } else if (name == i18nc("@label:textbox Bit rate label for track metadata view", "Bit Rate")) {
-        newRole = DataTypes::BitRateRole;
-    } else if (name == i18nc("@label:textbox Sample Rate label for track metadata view", "Sample Rate")) {
-        newRole = DataTypes::SampleRateRole;
-    } else if (name == i18nc("@label:textbox Lyrics label for track metadata view", "Lyrics")) {
-        newRole = DataTypes::LyricsRole;
-    } else if (name == i18nc("@label:textbox Duration label for track metadata view", "Duration")) {
-        newRole = DataTypes::DurationRole;
+    const auto modelIndex = this->index(index);
+    if (!modelIndex.isValid()) {
+        return;
     }
 
-    mDisplayData[newRole] = {};
-    mDisplayKeys.push_back(newRole);
+    const auto dataKey = mDisplayKeys[index];
+    if (mDisplayData.contains(dataKey)) {
+        mDisplayData.remove(dataKey);
+        Q_EMIT dataChanged(modelIndex, modelIndex);
+    }
+}
+
+void TrackMetadataModel::addDataByIndex(int index)
+{
+    const auto modelIndex = this->index(index);
+    if (!modelIndex.isValid()) {
+        return;
+    }
+
+    const auto newRole = mDisplayKeys[index];
+    if (!mDisplayData.contains(newRole)) {
+        mDisplayData.insert(newRole, QString());
+        Q_EMIT dataChanged(modelIndex, modelIndex);
+    }
 }
 
 QString TrackMetadataModel::nameFromRole(DataTypes::ColumnsRoles role)
@@ -621,9 +611,25 @@ void TrackMetadataModel::fetchLyrics()
         auto locker = QMutexLocker(&mFileScannerMutex);
         auto trackData = mFileScanner.scanOneFile(fileUrl);
         if (!trackData.lyrics().isEmpty()) {
-            return trackData.lyrics();
+            return std::make_pair(trackData.lyrics(), QString{});
         }
-        return QString{};
+        if (fileUrl.isLocalFile()) {
+            QFileInfo fileInfo(fileUrl.toLocalFile());
+            QDir lyricsDir(fileInfo.dir());
+            const QStringList lyricsFileExtensions = {u".lrc"_s, u".LRC"_s, u".txt"_s, u".TXT"_s};
+            QString lyricsFileName;
+
+            for (const QString &ext : lyricsFileExtensions) {
+                lyricsFileName = fileInfo.completeBaseName() + ext;
+                if (lyricsDir.exists(lyricsFileName)) {
+                    QFile file(lyricsDir.filePath(lyricsFileName));
+                    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+                        return std::make_pair(QString::fromLocal8Bit(file.readAll()), file.fileName());
+                    }
+                }
+            }
+        }
+        return std::make_pair(QString{}, QString{});
     });
 
     mLyricsValueWatcher.setFuture(lyricicsValue);
